# eslint-plugin-sonarjs [![npm version](https://badge.fury.io/js/eslint-plugin-sonarjs.svg)](https://badge.fury.io/js/eslint-plugin-sonarjs)

SonarJS rules for ESLint to help developers produce [Clean Code](https://www.sonarsource.com/solutions/clean-code/) by detecting bugs and suspicious patterns.

## Prerequisites

- Node.js (>=16.x) with ESLint 8.
- Node.js (^18.18.0 || ^20.9.0 || >=21) with ESLint 9.

## Usage

- If you don't have ESLint yet configured for your project, follow [these instructions](https://github.com/eslint/eslint#installation-and-usage).
- Install `eslint-plugin-sonarjs` using `npm` (or `yarn`) for your project or globally:

```sh
npm install eslint-plugin-sonarjs --save-dev # install for your project
npm install eslint-plugin-sonarjs -g         # or install globally
```

- Add `eslint-plugin-sonarjs` to the plugins of your ESLint config.

For ESLint 9: add `plugins` option to your `eslint.config.js` and include the recommended config to enable all rules:

```code
import sonarjs from "eslint-plugin-sonarjs";

[
  sonarjs.configs.recommended,
  {
    "plugins": {
      sonarjs,
    }
  }
]
```

For ESLint 8: add `plugins` option to your `.eslintrc` and `plugin:sonarjs/recommended-legacy` to the `extends` option to enable all recommended rules:

```json
{
  "plugins": ["sonarjs"],
  "extends": ["plugin:sonarjs/recommended-legacy"]
}
```

- or enable only some rules manually:

```json
{
  "rules": {
    "sonarjs/cognitive-complexity": "error",
    "sonarjs/no-identical-expressions": "error"
    // etc.
  }
}
```

- To allow each of the rules to fully perform, use `@typescript-eslint/parser` as a parser for ESLint and set the [parserOptions.project](https://github.com/typescript-eslint/typescript-eslint/tree/master/packages/parser#parseroptionsproject) option. This enables type checking which is beneficial or even essential for some rules.

## Available Configurations

This plugin provides only a `recommended` configuration. Almost all rules are activated in this profile with a few exceptions (check the `disabled` tag in the rules list). The `recommended` configuration activates rules with `error` severity.
We include a `recommended-legacy` configuration to be backward compatible with ESLint v8, with the same rule configuration..

## ESLint and Sonar

This plugin exposes to ESLint users all original JS/TS rules from Sonar products (aka [SonarJS](https://github.com/SonarSource/SonarJS)). We extracted the rules to be beneficial for the ESLint community.

This ESLint plugin does not contain all the rules from the SonarQube JS/TS analyzer. Aside of the rules available here, SonarQube uses rules from other ESLint plugins ([some](#eslint-rules) used as they are, [some others](#improved-eslint-rules) have been modified). We recommend installing those ESLint plugins and enabling those rules if you are looking for similar results to SonarQube using ESLint.

If you are a [SonarQube](https://www.sonarqube.org) or [SonarCloud](https://sonarcloud.io) user, to lint your code locally, we suggest using [SonarLint](https://www.sonarlint.org) IDE extension (available for VSCode, JetBrains IDEs and Eclipse). You can connect SonarLint to your SonarQube/SonarCloud project to synchronize rules configuration, issue statuses, etc.

## Rules

<!-- begin auto-generated rules list -->

💼 Configurations enabled in.\
✅ Set in the `recommended` configuration.\
🔧 Automatically fixable by the [`--fix` CLI option](https://eslint.org/docs/user-guide/command-line-interface#--fix).\
💡 Manually fixable by [editor suggestions](https://eslint.org/docs/latest/use/core-concepts#rule-suggestions).\
💭 Requires [type information](https://typescript-eslint.io/linting/typed-linting).\
❌ Deprecated.

| Name                                                                                                 | Description                                                                                                                    | 💼  | 🔧  | 💡  | 💭  | ❌  |
| :--------------------------------------------------------------------------------------------------- | :----------------------------------------------------------------------------------------------------------------------------- | :-- | :-- | :-- | :-- | :-- |
| [anchor-precedence](https://sonarsource.github.io/rspec/#/rspec/S5850/javascript)                    | Alternatives in regular expressions should be grouped when used with anchors                                                   | ✅  |     |     | 💭  |     |
| [argument-type](https://sonarsource.github.io/rspec/#/rspec/S3782/javascript)                        | Arguments to built-in functions should match documented types                                                                  | ✅  |     |     | 💭  |     |
| [arguments-order](https://sonarsource.github.io/rspec/#/rspec/S2234/javascript)                      | Parameters should be passed in the correct order                                                                               | ✅  |     |     | 💭  |     |
| [arguments-usage](https://sonarsource.github.io/rspec/#/rspec/S3513/javascript)                      | "arguments" should not be accessed directly                                                                                    |     |     |     |     |     |
| [array-callback-without-return](https://sonarsource.github.io/rspec/#/rspec/S3796/javascript)        | Callbacks of array methods should have return statements                                                                       | ✅  |     |     | 💭  |     |
| [array-constructor](https://sonarsource.github.io/rspec/#/rspec/S1528/javascript)                    | Array constructors should not be used                                                                                          |     |     | 💡  |     |     |
| [arrow-function-convention](https://sonarsource.github.io/rspec/#/rspec/S3524/javascript)            | Braces and parentheses should be used consistently with arrow functions                                                        |     |     |     |     |     |
| [assertions-in-tests](https://sonarsource.github.io/rspec/#/rspec/S2699/javascript)                  | Tests should include assertions                                                                                                | ✅  |     |     |     |     |
| [aws-apigateway-public-api](https://sonarsource.github.io/rspec/#/rspec/S6333/javascript)            | Creating public APIs is security-sensitive                                                                                     | ✅  |     |     |     |     |
| [aws-ec2-rds-dms-public](https://sonarsource.github.io/rspec/#/rspec/S6329/javascript)               | Allowing public network access to cloud resources is security-sensitive                                                        | ✅  |     |     |     |     |
| [aws-ec2-unencrypted-ebs-volume](https://sonarsource.github.io/rspec/#/rspec/S6275/javascript)       | Using unencrypted EBS volumes is security-sensitive                                                                            | ✅  |     |     |     |     |
| [aws-efs-unencrypted](https://sonarsource.github.io/rspec/#/rspec/S6332/javascript)                  | Using unencrypted EFS file systems is security-sensitive                                                                       | ✅  |     |     |     |     |
| [aws-iam-all-privileges](https://sonarsource.github.io/rspec/#/rspec/S6302/javascript)               | Policies granting all privileges are security-sensitive                                                                        | ✅  |     |     |     |     |
| [aws-iam-all-resources-accessible](https://sonarsource.github.io/rspec/#/rspec/S6304/javascript)     | Policies granting access to all resources of an account are security-sensitive                                                 |     |     |     |     |     |
| [aws-iam-privilege-escalation](https://sonarsource.github.io/rspec/#/rspec/S6317/javascript)         | AWS IAM policies should limit the scope of permissions given                                                                   | ✅  |     |     |     |     |
| [aws-iam-public-access](https://sonarsource.github.io/rspec/#/rspec/S6270/javascript)                | Policies authorizing public access to resources are security-sensitive                                                         | ✅  |     |     |     |     |
| [aws-opensearchservice-domain](https://sonarsource.github.io/rspec/#/rspec/S6308/javascript)         | Using unencrypted Elasticsearch domains is security-sensitive                                                                  | ✅  |     |     |     |     |
| [aws-rds-unencrypted-databases](https://sonarsource.github.io/rspec/#/rspec/S6303/javascript)        | Using unencrypted RDS DB resources is security-sensitive                                                                       | ✅  |     |     |     |     |
| [aws-restricted-ip-admin-access](https://sonarsource.github.io/rspec/#/rspec/S6321/javascript)       | Administration services access should be restricted to specific IP addresses                                                   | ✅  |     |     |     |     |
| [aws-s3-bucket-granted-access](https://sonarsource.github.io/rspec/#/rspec/S6265/javascript)         | Granting access to S3 buckets to all or authenticated users is security-sensitive                                              | ✅  |     |     |     |     |
| [aws-s3-bucket-insecure-http](https://sonarsource.github.io/rspec/#/rspec/S6249/javascript)          | Authorizing HTTP communications with S3 buckets is security-sensitive                                                          | ✅  |     |     |     |     |
| [aws-s3-bucket-public-access](https://sonarsource.github.io/rspec/#/rspec/S6281/javascript)          | Allowing public ACLs or policies on a S3 bucket is security-sensitive                                                          | ✅  |     |     |     |     |
| [aws-s3-bucket-server-encryption](https://sonarsource.github.io/rspec/#/rspec/S6245/javascript)      | Disabling server-side encryption of S3 buckets is security-sensitive                                                           |     |     |     |     | ❌  |
| [aws-s3-bucket-versioning](https://sonarsource.github.io/rspec/#/rspec/S6252/javascript)             | Disabling versioning of S3 buckets is security-sensitive                                                                       | ✅  |     |     |     |     |
| [aws-sagemaker-unencrypted-notebook](https://sonarsource.github.io/rspec/#/rspec/S6319/javascript)   | Using unencrypted SageMaker notebook instances is security-sensitive                                                           | ✅  |     |     |     |     |
| [aws-sns-unencrypted-topics](https://sonarsource.github.io/rspec/#/rspec/S6327/javascript)           | Using unencrypted SNS topics is security-sensitive                                                                             | ✅  |     |     |     |     |
| [aws-sqs-unencrypted-queue](https://sonarsource.github.io/rspec/#/rspec/S6330/javascript)            | Using unencrypted SQS queues is security-sensitive                                                                             | ✅  |     |     |     |     |
| [bitwise-operators](https://sonarsource.github.io/rspec/#/rspec/S1529/javascript)                    | Bitwise operators should not be used in boolean contexts                                                                       | ✅  |     |     |     |     |
| [block-scoped-var](https://sonarsource.github.io/rspec/#/rspec/S2392/javascript)                     | Variables should be used in the blocks where they are declared                                                                 | ✅  |     |     |     |     |
| [bool-param-default](https://sonarsource.github.io/rspec/#/rspec/S4798/javascript)                   | Optional boolean parameters should have default value                                                                          |     |     |     |     |     |
| [call-argument-line](https://sonarsource.github.io/rspec/#/rspec/S1472/javascript)                   | Function call arguments should not start on new lines                                                                          | ✅  |     |     |     |     |
| [certificate-transparency](https://sonarsource.github.io/rspec/#/rspec/S5742/javascript)             | Disabling Certificate Transparency monitoring is security-sensitive                                                            | ✅  |     |     |     |     |
| [chai-determinate-assertion](https://sonarsource.github.io/rspec/#/rspec/S6092/javascript)           | Chai assertions should have only one reason to succeed                                                                         | ✅  |     |     |     |     |
| [class-name](https://sonarsource.github.io/rspec/#/rspec/S101/javascript)                            | Class names should comply with a naming convention                                                                             | ✅  |     |     |     |     |
| [class-prototype](https://sonarsource.github.io/rspec/#/rspec/S3525/javascript)                      | Class methods should be used instead of "prototype" assignments                                                                |     |     |     | 💭  |     |
| [code-eval](https://sonarsource.github.io/rspec/#/rspec/S1523/javascript)                            | Dynamically executing code is security-sensitive                                                                               | ✅  |     |     |     |     |
| [cognitive-complexity](https://sonarsource.github.io/rspec/#/rspec/S3776/javascript)                 | Cognitive Complexity of functions should not be too high                                                                       | ✅  |     |     |     |     |
| [comma-or-logical-or-case](https://sonarsource.github.io/rspec/#/rspec/S3616/javascript)             | Comma and logical OR operators should not be used in switch cases                                                              | ✅  |     |     |     |     |
| [comment-regex](https://sonarsource.github.io/rspec/#/rspec/S124/javascript)                         | Track comments matching a regular expression                                                                                   |     |     |     |     |     |
| [concise-regex](https://sonarsource.github.io/rspec/#/rspec/S6353/javascript)                        | Regular expression quantifiers and character classes should be used concisely                                                  | ✅  |     |     | 💭  |     |
| [conditional-indentation](https://sonarsource.github.io/rspec/#/rspec/S3973/javascript)              | A conditionally executed single line should be denoted by indentation                                                          |     |     |     |     | ❌  |
| [confidential-information-logging](https://sonarsource.github.io/rspec/#/rspec/S5757/javascript)     | Allowing confidential information to be logged is security-sensitive                                                           | ✅  |     |     |     |     |
| [constructor-for-side-effects](https://sonarsource.github.io/rspec/#/rspec/S1848/javascript)         | Objects should not be created to be dropped immediately without being used                                                     | ✅  |     |     |     |     |
| [content-length](https://sonarsource.github.io/rspec/#/rspec/S5693/javascript)                       | Allowing requests with excessive content length is security-sensitive                                                          | ✅  |     |     |     |     |
| [content-security-policy](https://sonarsource.github.io/rspec/#/rspec/S5728/javascript)              | Disabling content security policy fetch directives is security-sensitive                                                       | ✅  |     |     |     |     |
| [cookie-no-httponly](https://sonarsource.github.io/rspec/#/rspec/S3330/javascript)                   | Creating cookies without the "HttpOnly" flag is security-sensitive                                                             | ✅  |     |     |     |     |
| [cookies](https://sonarsource.github.io/rspec/#/rspec/S2255/javascript)                              | Writing cookies is security-sensitive                                                                                          |     |     |     |     | ❌  |
| [cors](https://sonarsource.github.io/rspec/#/rspec/S5122/javascript)                                 | Having a permissive Cross-Origin Resource Sharing policy is security-sensitive                                                 | ✅  |     |     |     |     |
| [csrf](https://sonarsource.github.io/rspec/#/rspec/S4502/javascript)                                 | Disabling CSRF protections is security-sensitive                                                                               | ✅  |     |     |     |     |
| [cyclomatic-complexity](https://sonarsource.github.io/rspec/#/rspec/S1541/javascript)                | Cyclomatic Complexity of functions should not be too high                                                                      |     |     |     |     |     |
| [declarations-in-global-scope](https://sonarsource.github.io/rspec/#/rspec/S3798/javascript)         | Variables and functions should not be declared in the global scope                                                             |     |     |     |     |     |
| [deprecation](https://sonarsource.github.io/rspec/#/rspec/S1874/javascript)                          | Deprecated APIs should not be used                                                                                             | ✅  |     |     | 💭  |     |
| [destructuring-assignment-syntax](https://sonarsource.github.io/rspec/#/rspec/S3514/javascript)      | Destructuring syntax should be used for assignments                                                                            |     |     |     |     |     |
| [different-types-comparison](https://sonarsource.github.io/rspec/#/rspec/S3403/javascript)           | Strict equality operators should not be used with dissimilar types                                                             | ✅  |     | 💡  | 💭  |     |
| [disabled-auto-escaping](https://sonarsource.github.io/rspec/#/rspec/S5247/javascript)               | Disabling auto-escaping in template engines is security-sensitive                                                              | ✅  |     |     | 💭  |     |
| [disabled-resource-integrity](https://sonarsource.github.io/rspec/#/rspec/S5725/javascript)          | Using remote artifacts without integrity checks is security-sensitive                                                          | ✅  |     |     | 💭  |     |
| [disabled-timeout](https://sonarsource.github.io/rspec/#/rspec/S6080/javascript)                     | Disabling Mocha timeouts should be explicit                                                                                    | ✅  |     |     |     |     |
| [dns-prefetching](https://sonarsource.github.io/rspec/#/rspec/S5743/javascript)                      | Allowing browsers to perform DNS prefetching is security-sensitive                                                             |     |     |     |     | ❌  |
| [duplicates-in-character-class](https://sonarsource.github.io/rspec/#/rspec/S5869/javascript)        | Character classes in regular expressions should not contain the same character twice                                           | ✅  |     |     | 💭  |     |
| [elseif-without-else](https://sonarsource.github.io/rspec/#/rspec/S126/javascript)                   | "if ... else if" constructs should end with "else" clauses                                                                     |     |     |     |     |     |
| [empty-string-repetition](https://sonarsource.github.io/rspec/#/rspec/S5842/javascript)              | Repeated patterns in regular expressions should not match the empty string                                                     | ✅  |     |     | 💭  |     |
| [encryption](https://sonarsource.github.io/rspec/#/rspec/S4787/javascript)                           | Encrypting data is security-sensitive                                                                                          |     |     |     |     | ❌  |
| [encryption-secure-mode](https://sonarsource.github.io/rspec/#/rspec/S5542/javascript)               | Encryption algorithms should be used with secure mode and padding scheme                                                       | ✅  |     |     |     |     |
| [enforce-trailing-comma](https://sonarsource.github.io/rspec/#/rspec/S3723/javascript)               | Trailing commas should be used                                                                                                 |     | 🔧  |     |     | ❌  |
| [existing-groups](https://sonarsource.github.io/rspec/#/rspec/S6328/javascript)                      | Replacement strings should reference existing regular expression groups                                                        | ✅  |     |     | 💭  |     |
| [expression-complexity](https://sonarsource.github.io/rspec/#/rspec/S1067/javascript)                | Expressions should not be too complex                                                                                          |     |     |     |     |     |
| [file-header](https://sonarsource.github.io/rspec/#/rspec/S1451/javascript)                          | Track lack of copyright and license headers                                                                                    |     |     |     |     |     |
| [file-name-differ-from-class](https://sonarsource.github.io/rspec/#/rspec/S3317/javascript)          | Default export names and file names should match                                                                               |     |     |     |     |     |
| [file-permissions](https://sonarsource.github.io/rspec/#/rspec/S2612/javascript)                     | Setting loose POSIX file permissions is security-sensitive                                                                     | ✅  |     |     |     |     |
| [file-uploads](https://sonarsource.github.io/rspec/#/rspec/S2598/javascript)                         | File uploads should be restricted                                                                                              | ✅  |     |     |     |     |
| [fixme-tag](https://sonarsource.github.io/rspec/#/rspec/S1134/javascript)                            | Track uses of "FIXME" tags                                                                                                     | ✅  |     |     |     |     |
| [for-in](https://sonarsource.github.io/rspec/#/rspec/S1535/javascript)                               | "for...in" loops should filter properties before acting on them                                                                |     |     |     |     |     |
| [for-loop-increment-sign](https://sonarsource.github.io/rspec/#/rspec/S2251/javascript)              | A "for" loop update clause should move the counter in the right direction                                                      | ✅  |     |     |     |     |
| [frame-ancestors](https://sonarsource.github.io/rspec/#/rspec/S5732/javascript)                      | Disabling content security policy frame-ancestors directive is security-sensitive                                              | ✅  |     |     |     |     |
| [function-inside-loop](https://sonarsource.github.io/rspec/#/rspec/S1515/javascript)                 | Functions should not be defined inside loops                                                                                   | ✅  |     |     |     |     |
| [function-name](https://sonarsource.github.io/rspec/#/rspec/S100/javascript)                         | Function and method names should comply with a naming convention                                                               |     |     |     |     |     |
| [function-return-type](https://sonarsource.github.io/rspec/#/rspec/S3800/javascript)                 | Functions should always return the same type                                                                                   | ✅  |     |     | 💭  |     |
| [future-reserved-words](https://sonarsource.github.io/rspec/#/rspec/S1527/javascript)                | Future reserved words should not be used as identifiers                                                                        | ✅  |     |     |     |     |
| [generator-without-yield](https://sonarsource.github.io/rspec/#/rspec/S3531/javascript)              | Generators should explicitly "yield" a value                                                                                   | ✅  |     |     |     |     |
| [hashing](https://sonarsource.github.io/rspec/#/rspec/S4790/javascript)                              | Using weak hashing algorithms is security-sensitive                                                                            | ✅  |     |     |     |     |
| [hidden-files](https://sonarsource.github.io/rspec/#/rspec/S5691/javascript)                         | Statically serving hidden files is security-sensitive                                                                          | ✅  |     |     |     |     |
| [in-operator-type-error](https://sonarsource.github.io/rspec/#/rspec/S3785/javascript)               | "in" should not be used with primitive types                                                                                   | ✅  |     |     | 💭  |     |
| [inconsistent-function-call](https://sonarsource.github.io/rspec/#/rspec/S3686/javascript)           | Functions should be called consistently with or without "new"                                                                  | ✅  |     |     |     |     |
| [index-of-compare-to-positive-number](https://sonarsource.github.io/rspec/#/rspec/S2692/javascript)  | "indexOf" checks should not be for positive numbers                                                                            | ✅  |     |     | 💭  |     |
| [insecure-cookie](https://sonarsource.github.io/rspec/#/rspec/S2092/javascript)                      | Creating cookies without the "secure" flag is security-sensitive                                                               | ✅  |     |     |     |     |
| [insecure-jwt-token](https://sonarsource.github.io/rspec/#/rspec/S5659/javascript)                   | JWT should be signed and verified with strong cipher algorithms                                                                | ✅  |     |     |     |     |
| [inverted-assertion-arguments](https://sonarsource.github.io/rspec/#/rspec/S3415/javascript)         | Assertion arguments should be passed in the correct order                                                                      | ✅  |     | 💡  |     |     |
| [jsx-no-leaked-render](https://sonarsource.github.io/rspec/#/rspec/S6439/javascript)                 | React components should not render non-boolean condition values                                                                | ✅  |     | 💡  | 💭  |     |
| [label-position](https://sonarsource.github.io/rspec/#/rspec/S1439/javascript)                       | Only "while", "do", "for" and "switch" statements should be labelled                                                           | ✅  |     |     |     |     |
| [link-with-target-blank](https://sonarsource.github.io/rspec/#/rspec/S5148/javascript)               | Authorizing an opened window to access back to the originating window is security-sensitive                                    | ✅  |     |     |     |     |
| [max-lines](https://sonarsource.github.io/rspec/#/rspec/S104/javascript)                             | Files should not have too many lines of code                                                                                   |     |     |     |     |     |
| [max-lines-per-function](https://sonarsource.github.io/rspec/#/rspec/S138/javascript)                | Functions should not have too many lines of code                                                                               |     |     |     |     |     |
| [max-switch-cases](https://sonarsource.github.io/rspec/#/rspec/S1479/javascript)                     | "switch" statements should not have too many "case" clauses                                                                    | ✅  |     |     |     |     |
| [max-union-size](https://sonarsource.github.io/rspec/#/rspec/S4622/javascript)                       | Union types should not have too many elements                                                                                  |     |     |     |     |     |
| [misplaced-loop-counter](https://sonarsource.github.io/rspec/#/rspec/S1994/javascript)               | "for" loop increment clauses should modify the loops' counters                                                                 | ✅  |     |     |     |     |
| [nested-control-flow](https://sonarsource.github.io/rspec/#/rspec/S134/javascript)                   | Control flow statements "if", "for", "while", "switch" and "try" should not be nested too deeply                               |     |     |     |     |     |
| [new-operator-misuse](https://sonarsource.github.io/rspec/#/rspec/S2999/javascript)                  | "new" should only be used with functions and classes                                                                           | ✅  |     |     | 💭  |     |
| [no-all-duplicated-branches](https://sonarsource.github.io/rspec/#/rspec/S3923/javascript)           | All branches in a conditional structure should not have exactly the same implementation                                        | ✅  |     |     |     |     |
| [no-alphabetical-sort](https://sonarsource.github.io/rspec/#/rspec/S2871/javascript)                 | "Array.prototype.sort()" and "Array.prototype.toSorted()" should use a compare function                                        | ✅  |     | 💡  | 💭  |     |
| [no-angular-bypass-sanitization](https://sonarsource.github.io/rspec/#/rspec/S6268/javascript)       | Disabling Angular built-in sanitization is security-sensitive                                                                  | ✅  |     |     |     |     |
| [no-array-delete](https://sonarsource.github.io/rspec/#/rspec/S2870/javascript)                      | "delete" should not be used on arrays                                                                                          | ✅  |     |     | 💭  |     |
| [no-associative-arrays](https://sonarsource.github.io/rspec/#/rspec/S3579/javascript)                | Array indexes should be numeric                                                                                                | ✅  |     |     | 💭  |     |
| [no-async-constructor](https://sonarsource.github.io/rspec/#/rspec/S7059/javascript)                 | Constructors should not contain asynchronous operations                                                                        | ✅  |     |     |     |     |
| [no-built-in-override](https://sonarsource.github.io/rspec/#/rspec/S2424/javascript)                 | Built-in objects should not be overridden                                                                                      |     |     |     |     |     |
| [no-case-label-in-switch](https://sonarsource.github.io/rspec/#/rspec/S1219/javascript)              | "switch" statements should not contain non-case labels                                                                         | ✅  |     |     |     |     |
| [no-clear-text-protocols](https://sonarsource.github.io/rspec/#/rspec/S5332/javascript)              | Using clear-text protocols is security-sensitive                                                                               | ✅  |     |     |     |     |
| [no-code-after-done](https://sonarsource.github.io/rspec/#/rspec/S6079/javascript)                   | Tests should not execute any code after "done()" is called                                                                     | ✅  |     |     |     |     |
| [no-collapsible-if](https://sonarsource.github.io/rspec/#/rspec/S1066/javascript)                    | Mergeable "if" statements should be combined                                                                                   |     |     |     |     |     |
| [no-collection-size-mischeck](https://sonarsource.github.io/rspec/#/rspec/S3981/javascript)          | Collection size and array length comparisons should make sense                                                                 | ✅  |     | 💡  | 💭  |     |
| [no-commented-code](https://sonarsource.github.io/rspec/#/rspec/S125/javascript)                     | Sections of code should not be commented out                                                                                   | ✅  |     | 💡  |     |     |
| [no-control-regex](https://sonarsource.github.io/rspec/#/rspec/S6324/javascript)                     | Regular expressions should not contain control characters                                                                      | ✅  |     |     | 💭  |     |
| [no-dead-store](https://sonarsource.github.io/rspec/#/rspec/S1854/javascript)                        | Unused assignments should be removed                                                                                           | ✅  |     |     |     |     |
| [no-delete-var](https://sonarsource.github.io/rspec/#/rspec/S3001/javascript)                        | "delete" should be used only with object properties                                                                            | ✅  |     |     |     |     |
| [no-duplicate-in-composite](https://sonarsource.github.io/rspec/#/rspec/S4621/javascript)            | Union and intersection types should not include duplicated constituents                                                        | ✅  |     | 💡  |     |     |
| [no-duplicate-string](https://sonarsource.github.io/rspec/#/rspec/S1192/javascript)                  | String literals should not be duplicated                                                                                       |     |     |     |     |     |
| [no-duplicated-branches](https://sonarsource.github.io/rspec/#/rspec/S1871/javascript)               | Two branches in a conditional structure should not have exactly the same implementation                                        | ✅  |     |     |     |     |
| [no-element-overwrite](https://sonarsource.github.io/rspec/#/rspec/S4143/javascript)                 | Collection elements should not be replaced unconditionally                                                                     | ✅  |     |     |     |     |
| [no-empty-after-reluctant](https://sonarsource.github.io/rspec/#/rspec/S6019/javascript)             | Reluctant quantifiers in regular expressions should be followed by an expression that can't match the empty string             | ✅  |     |     | 💭  |     |
| [no-empty-alternatives](https://sonarsource.github.io/rspec/#/rspec/S6323/javascript)                | Alternation in regular expressions should not contain empty alternatives                                                       | ✅  |     |     | 💭  |     |
| [no-empty-character-class](https://sonarsource.github.io/rspec/#/rspec/S2639/javascript)             | Empty character classes should not be used                                                                                     | ✅  |     |     | 💭  |     |
| [no-empty-collection](https://sonarsource.github.io/rspec/#/rspec/S4158/javascript)                  | Empty collections should not be accessed or iterated                                                                           | ✅  |     |     |     |     |
| [no-empty-group](https://sonarsource.github.io/rspec/#/rspec/S6331/javascript)                       | Regular expressions should not contain empty groups                                                                            | ✅  |     |     | 💭  |     |
| [no-empty-test-file](https://sonarsource.github.io/rspec/#/rspec/S2187/javascript)                   | Test files should contain at least one test case                                                                               | ✅  |     |     |     |     |
| [no-equals-in-for-termination](https://sonarsource.github.io/rspec/#/rspec/S888/javascript)          | Equality operators should not be used in "for" loop termination conditions                                                     | ✅  |     |     |     |     |
| [no-exclusive-tests](https://sonarsource.github.io/rspec/#/rspec/S6426/javascript)                   | Exclusive tests should not be commited to version control                                                                      | ✅  |     | 💡  |     |     |
| [no-extra-arguments](https://sonarsource.github.io/rspec/#/rspec/S930/javascript)                    | Function calls should not pass extra arguments                                                                                 | ✅  |     |     |     |     |
| [no-fallthrough](https://sonarsource.github.io/rspec/#/rspec/S128/javascript)                        | Switch cases should end with an unconditional "break" statement                                                                | ✅  |     |     |     |     |
| [no-for-in-iterable](https://sonarsource.github.io/rspec/#/rspec/S4139/javascript)                   | "for in" should not be used with iterables                                                                                     |     |     |     | 💭  |     |
| [no-function-declaration-in-block](https://sonarsource.github.io/rspec/#/rspec/S1530/javascript)     | Function declarations should not be made within blocks                                                                         |     |     |     |     |     |
| [no-global-this](https://sonarsource.github.io/rspec/#/rspec/S2990/javascript)                       | The global "this" object should not be used                                                                                    | ✅  |     | 💡  |     |     |
| [no-globals-shadowing](https://sonarsource.github.io/rspec/#/rspec/S2137/javascript)                 | Special identifiers should not be bound or assigned                                                                            | ✅  |     |     |     |     |
| [no-gratuitous-expressions](https://sonarsource.github.io/rspec/#/rspec/S2589/javascript)            | Boolean expressions should not be gratuitous                                                                                   | ✅  |     |     |     |     |
| [no-hardcoded-ip](https://sonarsource.github.io/rspec/#/rspec/S1313/javascript)                      | Using hardcoded IP addresses is security-sensitive                                                                             | ✅  |     |     |     |     |
| [no-hardcoded-passwords](https://sonarsource.github.io/rspec/#/rspec/S2068/javascript)               | Hard-coded passwords are security-sensitive                                                                                    | ✅  |     |     |     |     |
| [no-hardcoded-secrets](https://sonarsource.github.io/rspec/#/rspec/S6418/javascript)                 | Hard-coded secrets are security-sensitive                                                                                      | ✅  |     |     |     |     |
| [no-hook-setter-in-body](https://sonarsource.github.io/rspec/#/rspec/S6442/javascript)               | React's useState hook should not be used directly in the render function or body of a component                                | ✅  |     |     |     |     |
| [no-identical-conditions](https://sonarsource.github.io/rspec/#/rspec/S1862/javascript)              | "if/else if" chains and "switch" cases should not have the same condition                                                      | ✅  |     |     |     |     |
| [no-identical-expressions](https://sonarsource.github.io/rspec/#/rspec/S1764/javascript)             | Identical expressions should not be used on both sides of a binary operator                                                    | ✅  |     |     |     |     |
| [no-identical-functions](https://sonarsource.github.io/rspec/#/rspec/S4144/javascript)               | Functions should not have identical implementations                                                                            | ✅  |     |     |     |     |
| [no-ignored-exceptions](https://sonarsource.github.io/rspec/#/rspec/S2486/javascript)                | Exceptions should not be ignored                                                                                               | ✅  |     |     |     |     |
| [no-ignored-return](https://sonarsource.github.io/rspec/#/rspec/S2201/javascript)                    | Return values from functions without side effects should not be ignored                                                        | ✅  |     |     | 💭  |     |
| [no-implicit-dependencies](https://sonarsource.github.io/rspec/#/rspec/S4328/javascript)             | Dependencies should be explicit                                                                                                |     |     |     |     |     |
| [no-implicit-global](https://sonarsource.github.io/rspec/#/rspec/S2703/javascript)                   | Variables should be declared explicitly                                                                                        | ✅  |     |     |     |     |
| [no-in-misuse](https://sonarsource.github.io/rspec/#/rspec/S4619/javascript)                         | "in" should not be used on arrays                                                                                              | ✅  |     | 💡  | 💭  |     |
| [no-incomplete-assertions](https://sonarsource.github.io/rspec/#/rspec/S2970/javascript)             | Assertions should be complete                                                                                                  | ✅  |     |     |     |     |
| [no-inconsistent-returns](https://sonarsource.github.io/rspec/#/rspec/S3801/javascript)              | Functions should use "return" consistently                                                                                     |     |     |     |     |     |
| [no-incorrect-string-concat](https://sonarsource.github.io/rspec/#/rspec/S3402/javascript)           | Strings and non-strings should not be added                                                                                    |     |     |     | 💭  |     |
| [no-internal-api-use](https://sonarsource.github.io/rspec/#/rspec/S6627/javascript)                  | Users should not use internal APIs                                                                                             | ✅  |     |     |     |     |
| [no-intrusive-permissions](https://sonarsource.github.io/rspec/#/rspec/S5604/javascript)             | Using intrusive permissions is security-sensitive                                                                              | ✅  |     |     |     |     |
| [no-invalid-await](https://sonarsource.github.io/rspec/#/rspec/S4123/javascript)                     | "await" should only be used with promises                                                                                      | ✅  |     |     | 💭  |     |
| [no-invalid-regexp](https://sonarsource.github.io/rspec/#/rspec/S5856/javascript)                    | Regular expressions should be syntactically valid                                                                              | ✅  |     |     | 💭  |     |
| [no-invariant-returns](https://sonarsource.github.io/rspec/#/rspec/S3516/javascript)                 | Function returns should not be invariant                                                                                       | ✅  |     |     |     |     |
| [no-inverted-boolean-check](https://sonarsource.github.io/rspec/#/rspec/S1940/javascript)            | Boolean checks should not be inverted                                                                                          | ✅  | 🔧  | 💡  |     |     |
| [no-ip-forward](https://sonarsource.github.io/rspec/#/rspec/S5759/javascript)                        | Forwarding client IP address is security-sensitive                                                                             | ✅  |     |     |     |     |
| [no-labels](https://sonarsource.github.io/rspec/#/rspec/S1119/javascript)                            | Labels should not be used                                                                                                      | ✅  |     |     |     |     |
| [no-literal-call](https://sonarsource.github.io/rspec/#/rspec/S6958/javascript)                      | Literals should not be used as functions                                                                                       | ✅  |     |     |     |     |
| [no-mime-sniff](https://sonarsource.github.io/rspec/#/rspec/S5734/javascript)                        | Allowing browsers to sniff MIME types is security-sensitive                                                                    | ✅  |     |     |     |     |
| [no-misleading-array-reverse](https://sonarsource.github.io/rspec/#/rspec/S4043/javascript)          | Array-mutating methods should not be used misleadingly                                                                         | ✅  |     | 💡  | 💭  |     |
| [no-misleading-character-class](https://sonarsource.github.io/rspec/#/rspec/S5868/javascript)        | Unicode Grapheme Clusters should be avoided inside regex character classes                                                     | ✅  |     | 💡  | 💭  |     |
| [no-mixed-content](https://sonarsource.github.io/rspec/#/rspec/S5730/javascript)                     | Allowing mixed-content is security-sensitive                                                                                   | ✅  |     |     |     |     |
| [no-nested-assignment](https://sonarsource.github.io/rspec/#/rspec/S1121/javascript)                 | Assignments should not be made from within sub-expressions                                                                     | ✅  |     |     |     |     |
| [no-nested-conditional](https://sonarsource.github.io/rspec/#/rspec/S3358/javascript)                | Ternary operators should not be nested                                                                                         | ✅  |     |     |     |     |
| [no-nested-functions](https://sonarsource.github.io/rspec/#/rspec/S2004/javascript)                  | Functions should not be nested too deeply                                                                                      | ✅  |     |     |     |     |
| [no-nested-incdec](https://sonarsource.github.io/rspec/#/rspec/S881/javascript)                      | Increment (++) and decrement (--) operators should not be used in a method call or mixed with other operators in an expression |     |     |     |     |     |
| [no-nested-switch](https://sonarsource.github.io/rspec/#/rspec/S1821/javascript)                     | "switch" statements should not be nested                                                                                       |     |     |     |     |     |
| [no-nested-template-literals](https://sonarsource.github.io/rspec/#/rspec/S4624/javascript)          | Template literals should not be nested                                                                                         | ✅  |     |     |     |     |
| [no-one-iteration-loop](https://sonarsource.github.io/rspec/#/rspec/S1751/javascript)                | Loops with at most one iteration should be refactored                                                                          | ✅  |     |     |     |     |
| [no-os-command-from-path](https://sonarsource.github.io/rspec/#/rspec/S4036/javascript)              | Searching OS commands in PATH is security-sensitive                                                                            | ✅  |     |     |     |     |
| [no-parameter-reassignment](https://sonarsource.github.io/rspec/#/rspec/S1226/javascript)            | Initial values of parameters, caught exceptions, and loop variables should not be ignored                                      | ✅  |     |     |     |     |
| [no-primitive-wrappers](https://sonarsource.github.io/rspec/#/rspec/S1533/javascript)                | Wrapper objects should not be used for primitive types                                                                         | ✅  |     | 💡  |     |     |
| [no-redundant-assignments](https://sonarsource.github.io/rspec/#/rspec/S4165/javascript)             | Assignments should not be redundant                                                                                            | ✅  |     |     |     |     |
| [no-redundant-boolean](https://sonarsource.github.io/rspec/#/rspec/S1125/javascript)                 | Boolean literals should not be used in comparisons                                                                             | ✅  |     |     |     |     |
| [no-redundant-jump](https://sonarsource.github.io/rspec/#/rspec/S3626/javascript)                    | Jump statements should not be redundant                                                                                        | ✅  |     | 💡  |     |     |
| [no-redundant-optional](https://sonarsource.github.io/rspec/#/rspec/S4782/javascript)                | Optional property declarations should not use both '?' and 'undefined' syntax                                                  | ✅  |     | 💡  | 💭  |     |
| [no-redundant-parentheses](https://sonarsource.github.io/rspec/#/rspec/S1110/javascript)             | Redundant pairs of parentheses should be removed                                                                               |     |     | 💡  |     | ❌  |
| [no-reference-error](https://sonarsource.github.io/rspec/#/rspec/S3827/javascript)                   | Variables should be defined before being used                                                                                  |     |     |     |     |     |
| [no-referrer-policy](https://sonarsource.github.io/rspec/#/rspec/S5736/javascript)                   | Disabling strict HTTP no-referrer policy is security-sensitive                                                                 | ✅  |     |     |     |     |
| [no-regex-spaces](https://sonarsource.github.io/rspec/#/rspec/S6326/javascript)                      | Regular expressions should not contain multiple spaces                                                                         | ✅  |     | 💡  | 💭  |     |
| [no-require-or-define](https://sonarsource.github.io/rspec/#/rspec/S3533/javascript)                 | "import" should be used to include external code                                                                               |     |     |     | 💭  |     |
| [no-return-type-any](https://sonarsource.github.io/rspec/#/rspec/S4324/javascript)                   | Primitive return types should be used                                                                                          |     |     |     | 💭  |     |
| [no-same-argument-assert](https://sonarsource.github.io/rspec/#/rspec/S5863/javascript)              | Assertions should not be given twice the same argument                                                                         | ✅  |     |     |     |     |
| [no-same-line-conditional](https://sonarsource.github.io/rspec/#/rspec/S3972/javascript)             | Conditionals should start on new lines                                                                                         | ✅  |     | 💡  |     |     |
| [no-selector-parameter](https://sonarsource.github.io/rspec/#/rspec/S2301/javascript)                | Methods should not contain selector parameters                                                                                 | ✅  |     |     | 💭  |     |
| [no-skipped-tests](https://sonarsource.github.io/rspec/#/rspec/S1607/javascript)                     | Tests should not be skipped without providing a reason                                                                         | ✅  |     |     |     |     |
| [no-small-switch](https://sonarsource.github.io/rspec/#/rspec/S1301/javascript)                      | "if" statements should be preferred over "switch" when simpler                                                                 | ✅  |     |     |     |     |
| [no-sonar-comments](https://sonarsource.github.io/rspec/#/rspec/S1291/javascript)                    | Track uses of "NOSONAR" comments                                                                                               |     |     |     |     |     |
| [no-tab](https://sonarsource.github.io/rspec/#/rspec/S105/javascript)                                | Tabulation characters should not be used                                                                                       |     |     |     |     | ❌  |
| [no-table-as-layout](https://sonarsource.github.io/rspec/#/rspec/S5257/javascript)                   | HTML "&lt;table&gt;" should not be used for layout purposes                                                                    | ✅  |     |     |     |     |
| [no-try-promise](https://sonarsource.github.io/rspec/#/rspec/S4822/javascript)                       | Promise rejections should not be caught by "try" blocks                                                                        | ✅  |     |     | 💭  |     |
| [no-undefined-argument](https://sonarsource.github.io/rspec/#/rspec/S4623/javascript)                | "undefined" should not be passed as the value of optional parameters                                                           | ✅  |     | 💡  | 💭  |     |
| [no-undefined-assignment](https://sonarsource.github.io/rspec/#/rspec/S2138/javascript)              | "undefined" should not be assigned                                                                                             |     |     |     |     |     |
| [no-unenclosed-multiline-block](https://sonarsource.github.io/rspec/#/rspec/S2681/javascript)        | Multiline blocks should be enclosed in curly braces                                                                            | ✅  |     |     |     |     |
| [no-uniq-key](https://sonarsource.github.io/rspec/#/rspec/S6486/javascript)                          | JSX list components keys should match up between renders                                                                       | ✅  |     |     |     |     |
| [no-unsafe-unzip](https://sonarsource.github.io/rspec/#/rspec/S5042/javascript)                      | Expanding archive files without controlling resource consumption is security-sensitive                                         | ✅  |     |     |     |     |
| [no-unthrown-error](https://sonarsource.github.io/rspec/#/rspec/S3984/javascript)                    | Errors should not be created without being thrown                                                                              | ✅  |     | 💡  |     |     |
| [no-unused-collection](https://sonarsource.github.io/rspec/#/rspec/S4030/javascript)                 | Collection contents should be used                                                                                             | ✅  |     |     |     |     |
| [no-unused-function-argument](https://sonarsource.github.io/rspec/#/rspec/S1172/javascript)          | Unused function parameters should be removed                                                                                   |     |     | 💡  |     |     |
| [no-unused-vars](https://sonarsource.github.io/rspec/#/rspec/S1481/javascript)                       | Unused local variables and functions should be removed                                                                         | ✅  |     |     |     |     |
| [no-use-of-empty-return-value](https://sonarsource.github.io/rspec/#/rspec/S3699/javascript)         | The return value of void functions should not be used                                                                          | ✅  |     |     |     |     |
| [no-useless-catch](https://sonarsource.github.io/rspec/#/rspec/S2737/javascript)                     | "catch" clauses should do more than rethrow                                                                                    | ✅  |     |     |     |     |
| [no-useless-increment](https://sonarsource.github.io/rspec/#/rspec/S2123/javascript)                 | Values should not be uselessly incremented                                                                                     | ✅  |     |     |     |     |
| [no-useless-intersection](https://sonarsource.github.io/rspec/#/rspec/S4335/javascript)              | Type intersections should use meaningful types                                                                                 | ✅  |     |     | 💭  |     |
| [no-useless-react-setstate](https://sonarsource.github.io/rspec/#/rspec/S6443/javascript)            | React state setter function should not be called with its matching state variable                                              | ✅  |     |     |     |     |
| [no-variable-usage-before-declaration](https://sonarsource.github.io/rspec/#/rspec/S1526/javascript) | Variables declared with "var" should be declared before they are used                                                          |     |     |     |     |     |
| [no-vue-bypass-sanitization](https://sonarsource.github.io/rspec/#/rspec/S6299/javascript)           | Disabling Vue.js built-in escaping is security-sensitive                                                                       | ✅  |     |     |     |     |
| [no-weak-cipher](https://sonarsource.github.io/rspec/#/rspec/S5547/javascript)                       | Cipher algorithms should be robust                                                                                             | ✅  |     |     |     |     |
| [no-weak-keys](https://sonarsource.github.io/rspec/#/rspec/S4426/javascript)                         | Cryptographic keys should be robust                                                                                            | ✅  |     |     |     |     |
| [no-wildcard-import](https://sonarsource.github.io/rspec/#/rspec/S2208/javascript)                   | Wildcard imports should not be used                                                                                            |     |     |     |     |     |
| [non-existent-operator](https://sonarsource.github.io/rspec/#/rspec/S2757/javascript)                | Non-existent operators '=+', '=-' and '=!' should not be used                                                                  | ✅  |     | 💡  |     |     |
| [non-number-in-arithmetic-expression](https://sonarsource.github.io/rspec/#/rspec/S3760/javascript)  | Arithmetic operators should only have numbers as operands                                                                      |     |     |     | 💭  |     |
| [null-dereference](https://sonarsource.github.io/rspec/#/rspec/S2259/javascript)                     | Properties of variables with "null" or "undefined" values should not be accessed                                               | ✅  |     |     | 💭  |     |
| [object-alt-content](https://sonarsource.github.io/rspec/#/rspec/S5264/javascript)                   | "<object>" tags should provide an alternative content                                                                          | ✅  |     |     |     |     |
| [operation-returning-nan](https://sonarsource.github.io/rspec/#/rspec/S3757/javascript)              | Arithmetic operations should not result in "NaN"                                                                               |     |     |     | 💭  |     |
| [os-command](https://sonarsource.github.io/rspec/#/rspec/S4721/javascript)                           | Using shell interpreter when executing OS commands is security-sensitive                                                       | ✅  |     |     |     |     |
| [post-message](https://sonarsource.github.io/rspec/#/rspec/S2819/javascript)                         | Origins should be verified during cross-origin communications                                                                  | ✅  |     |     | 💭  |     |
| [prefer-default-last](https://sonarsource.github.io/rspec/#/rspec/S4524/javascript)                  | "default" clauses should be last                                                                                               | ✅  |     |     |     |     |
| [prefer-immediate-return](https://sonarsource.github.io/rspec/#/rspec/S1488/javascript)              | Local variables should not be declared and then immediately returned or thrown                                                 |     | 🔧  |     |     |     |
| [prefer-object-literal](https://sonarsource.github.io/rspec/#/rspec/S2428/javascript)                | Object literal syntax should be used                                                                                           |     |     |     |     |     |
| [prefer-promise-shorthand](https://sonarsource.github.io/rspec/#/rspec/S4634/javascript)             | Shorthand promises should be used                                                                                              | ✅  |     | 💡  |     |     |
| [prefer-read-only-props](https://sonarsource.github.io/rspec/#/rspec/S6759/javascript)               | React props should be read-only                                                                                                | ✅  |     | 💡  | 💭  |     |
| [prefer-regexp-exec](https://sonarsource.github.io/rspec/#/rspec/S6594/javascript)                   | "RegExp.exec()" should be preferred over "String.match()"                                                                      | ✅  |     | 💡  | 💭  |     |
| [prefer-single-boolean-return](https://sonarsource.github.io/rspec/#/rspec/S1126/javascript)         | Return of boolean expressions should not be wrapped into an "if-then-else" statement                                           | ✅  |     | 💡  |     |     |
| [prefer-type-guard](https://sonarsource.github.io/rspec/#/rspec/S4322/javascript)                    | Type predicates should be used                                                                                                 | ✅  |     | 💡  |     |     |
| [prefer-while](https://sonarsource.github.io/rspec/#/rspec/S1264/javascript)                         | A "while" loop should be used instead of a "for" loop                                                                          | ✅  | 🔧  |     |     |     |
| [process-argv](https://sonarsource.github.io/rspec/#/rspec/S4823/javascript)                         | Using command line arguments is security-sensitive                                                                             |     |     |     |     | ❌  |
| [production-debug](https://sonarsource.github.io/rspec/#/rspec/S4507/javascript)                     | Delivering code in production with debug features activated is security-sensitive                                              | ✅  |     |     |     |     |
| [pseudo-random](https://sonarsource.github.io/rspec/#/rspec/S2245/javascript)                        | Using pseudorandom number generators (PRNGs) is security-sensitive                                                             | ✅  |     |     |     |     |
| [public-static-readonly](https://sonarsource.github.io/rspec/#/rspec/S1444/javascript)               | Public "static" fields should be read-only                                                                                     | ✅  |     | 💡  |     |     |
| [publicly-writable-directories](https://sonarsource.github.io/rspec/#/rspec/S5443/javascript)        | Using publicly writable directories is security-sensitive                                                                      | ✅  |     |     |     |     |
| [reduce-initial-value](https://sonarsource.github.io/rspec/#/rspec/S6959/javascript)                 | "Array.reduce()" calls should include an initial value                                                                         | ✅  |     |     | 💭  |     |
| [redundant-type-aliases](https://sonarsource.github.io/rspec/#/rspec/S6564/javascript)               | Redundant type aliases should not be used                                                                                      | ✅  |     |     |     |     |
| [regex-complexity](https://sonarsource.github.io/rspec/#/rspec/S5843/javascript)                     | Regular expressions should not be too complicated                                                                              | ✅  |     |     | 💭  |     |
| [regular-expr](https://sonarsource.github.io/rspec/#/rspec/S4784/javascript)                         | Using regular expressions is security-sensitive                                                                                |     |     |     |     | ❌  |
| [session-regeneration](https://sonarsource.github.io/rspec/#/rspec/S5876/javascript)                 | A new session should be created during user authentication                                                                     | ✅  |     |     |     |     |
| [shorthand-property-grouping](https://sonarsource.github.io/rspec/#/rspec/S3499/javascript)          | Shorthand object properties should be grouped at the beginning or end of an object declaration                                 |     |     |     |     |     |
| [single-char-in-character-classes](https://sonarsource.github.io/rspec/#/rspec/S6397/javascript)     | Character classes in regular expressions should not contain only one character                                                 | ✅  |     |     | 💭  |     |
| [single-character-alternation](https://sonarsource.github.io/rspec/#/rspec/S6035/javascript)         | Single-character alternations in regular expressions should be replaced with character classes                                 | ✅  |     |     | 💭  |     |
| [slow-regex](https://sonarsource.github.io/rspec/#/rspec/S5852/javascript)                           | Using slow regular expressions is security-sensitive                                                                           | ✅  |     |     | 💭  |     |
| [sockets](https://sonarsource.github.io/rspec/#/rspec/S4818/javascript)                              | Using Sockets is security-sensitive                                                                                            |     |     |     |     | ❌  |
| [sql-queries](https://sonarsource.github.io/rspec/#/rspec/S2077/javascript)                          | Formatting SQL queries is security-sensitive                                                                                   | ✅  |     |     |     |     |
| [stable-tests](https://sonarsource.github.io/rspec/#/rspec/S5973/javascript)                         | Tests should be stable                                                                                                         | ✅  |     |     |     |     |
| [standard-input](https://sonarsource.github.io/rspec/#/rspec/S4829/javascript)                       | Reading the Standard Input is security-sensitive                                                                               |     |     |     |     | ❌  |
| [stateful-regex](https://sonarsource.github.io/rspec/#/rspec/S6351/javascript)                       | Regular expressions with the global flag should be used with caution                                                           | ✅  |     |     |     |     |
| [strict-transport-security](https://sonarsource.github.io/rspec/#/rspec/S5739/javascript)            | Disabling Strict-Transport-Security policy is security-sensitive                                                               | ✅  |     |     |     |     |
| [strings-comparison](https://sonarsource.github.io/rspec/#/rspec/S3003/javascript)                   | Comparison operators should not be used with strings                                                                           |     |     |     | 💭  |     |
| [super-invocation](https://sonarsource.github.io/rspec/#/rspec/S3854/javascript)                     | "super()" should be invoked appropriately                                                                                      | ✅  |     |     |     |     |
| [table-header](https://sonarsource.github.io/rspec/#/rspec/S5256/javascript)                         | Tables should have headers                                                                                                     | ✅  |     |     |     |     |
| [table-header-reference](https://sonarsource.github.io/rspec/#/rspec/S5260/javascript)               | Table cells should reference their headers                                                                                     | ✅  |     |     |     |     |
| [test-check-exception](https://sonarsource.github.io/rspec/#/rspec/S5958/javascript)                 | Tests should check which exception is thrown                                                                                   | ✅  |     |     |     |     |
| [todo-tag](https://sonarsource.github.io/rspec/#/rspec/S1135/javascript)                             | Track uses of "TODO" tags                                                                                                      | ✅  |     |     |     |     |
| [too-many-break-or-continue-in-loop](https://sonarsource.github.io/rspec/#/rspec/S135/javascript)    | Loops should not contain more than a single "break" or "continue" statement                                                    |     |     |     |     |     |
| [unicode-aware-regex](https://sonarsource.github.io/rspec/#/rspec/S5867/javascript)                  | Regular expressions using Unicode character classes or property escapes should enable the unicode flag                         |     |     |     | 💭  |     |
| [unused-import](https://sonarsource.github.io/rspec/#/rspec/S1128/javascript)                        | Unnecessary imports should be removed                                                                                          | ✅  |     | 💡  | 💭  |     |
| [unused-named-groups](https://sonarsource.github.io/rspec/#/rspec/S5860/javascript)                  | Names of regular expressions named groups should be used                                                                       | ✅  |     |     | 💭  |     |
| [unverified-certificate](https://sonarsource.github.io/rspec/#/rspec/S4830/javascript)               | Server certificates should be verified during SSL/TLS connections                                                              | ✅  |     |     |     |     |
| [unverified-hostname](https://sonarsource.github.io/rspec/#/rspec/S5527/javascript)                  | Server hostnames should be verified during SSL/TLS connections                                                                 | ✅  |     |     |     |     |
| [updated-const-var](https://sonarsource.github.io/rspec/#/rspec/S3500/javascript)                    | "const" variables should not be reassigned                                                                                     | ✅  |     |     |     |     |
| [updated-loop-counter](https://sonarsource.github.io/rspec/#/rspec/S2310/javascript)                 | Loop counters should not be assigned within the loop body                                                                      | ✅  |     |     |     |     |
| [use-type-alias](https://sonarsource.github.io/rspec/#/rspec/S4323/javascript)                       | Type aliases should be used                                                                                                    | ✅  |     |     |     |     |
| [useless-string-operation](https://sonarsource.github.io/rspec/#/rspec/S1154/javascript)             | Results of operations on strings should not be ignored                                                                         |     |     |     | 💭  | ❌  |
| [values-not-convertible-to-numbers](https://sonarsource.github.io/rspec/#/rspec/S3758/javascript)    | Values not convertible to numbers should not be used in numeric comparisons                                                    |     |     |     | 💭  |     |
| [variable-name](https://sonarsource.github.io/rspec/#/rspec/S117/javascript)                         | Variable, property and parameter names should comply with a naming convention                                                  |     |     |     |     |     |
| [void-use](https://sonarsource.github.io/rspec/#/rspec/S3735/javascript)                             | "void" should not be used                                                                                                      | ✅  |     |     | 💭  |     |
| [weak-ssl](https://sonarsource.github.io/rspec/#/rspec/S4423/javascript)                             | Weak SSL/TLS protocols should not be used                                                                                      | ✅  |     |     |     |     |
| [web-sql-database](https://sonarsource.github.io/rspec/#/rspec/S2817/javascript)                     | Web SQL databases should not be used                                                                                           |     |     |     | 💭  | ❌  |
| [x-powered-by](https://sonarsource.github.io/rspec/#/rspec/S5689/javascript)                         | Disclosing fingerprints from web application technologies is security-sensitive                                                | ✅  |     |     |     |     |
| [xml-parser-xxe](https://sonarsource.github.io/rspec/#/rspec/S2755/javascript)                       | XML parsers should not be vulnerable to XXE attacks                                                                            | ✅  |     |     |     |     |
| [xpath](https://sonarsource.github.io/rspec/#/rspec/S4817/javascript)                                | Executing XPath expressions is security-sensitive                                                                              |     |     |     |     | ❌  |

<!-- end auto-generated rules list -->

## ESLint rules

Some rules are not shipped in this ESLint plugin to avoid duplication with already existing rules from ESLint core and third-party ESLint plugins.

The following rules are used directly, without modifying the original behavior:

<!--- start external rules -->

| SonarJS rule ID                                                       | Rule implemented by                                                                                                                                                                            |
| :-------------------------------------------------------------------- | :--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| [S103](https://sonarsource.github.io/rspec/#/rspec/S103/javascript)   | [eslint/max-len](https://eslint.org/docs/latest/rules/max-len)                                                                                                                                 |
| [S106](https://sonarsource.github.io/rspec/#/rspec/S106/javascript)   | [eslint/no-console](https://eslint.org/docs/latest/rules/no-console)                                                                                                                           |
| [S108](https://sonarsource.github.io/rspec/#/rspec/S108/javascript)   | [eslint/no-empty](https://eslint.org/docs/latest/rules/no-empty)                                                                                                                               |
| [S113](https://sonarsource.github.io/rspec/#/rspec/S113/javascript)   | [eslint/eol-last](https://eslint.org/docs/latest/rules/eol-last)                                                                                                                               |
| [S121](https://sonarsource.github.io/rspec/#/rspec/S121/javascript)   | [eslint/curly](https://eslint.org/docs/latest/rules/curly)                                                                                                                                     |
| [S122](https://sonarsource.github.io/rspec/#/rspec/S122/javascript)   | [eslint/max-statements-per-line](https://eslint.org/docs/latest/rules/max-statements-per-line)                                                                                                 |
| [S139](https://sonarsource.github.io/rspec/#/rspec/S139/javascript)   | [eslint/line-comment-position](https://eslint.org/docs/latest/rules/line-comment-position)                                                                                                     |
| [S878](https://sonarsource.github.io/rspec/#/rspec/S878/javascript)   | [eslint/no-sequences](https://eslint.org/docs/latest/rules/no-sequences)                                                                                                                       |
| [S909](https://sonarsource.github.io/rspec/#/rspec/S909/javascript)   | [eslint/no-continue](https://eslint.org/docs/latest/rules/no-continue)                                                                                                                         |
| [S1090](https://sonarsource.github.io/rspec/#/rspec/S1090/javascript) | [jsx-a11y/iframe-has-title](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/iframe-has-title.md)                                                                     |
| [S1117](https://sonarsource.github.io/rspec/#/rspec/S1117/javascript) | [typescript-eslint/no-shadow](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-shadow.mdx)                                             |
| [S1131](https://sonarsource.github.io/rspec/#/rspec/S1131/javascript) | [eslint/no-trailing-spaces](https://eslint.org/docs/latest/rules/no-trailing-spaces)                                                                                                           |
| [S1143](https://sonarsource.github.io/rspec/#/rspec/S1143/javascript) | [eslint/no-unsafe-finally](https://eslint.org/docs/latest/rules/no-unsafe-finally)                                                                                                             |
| [S1199](https://sonarsource.github.io/rspec/#/rspec/S1199/javascript) | [eslint/no-lone-blocks](https://eslint.org/docs/latest/rules/no-lone-blocks)                                                                                                                   |
| [S1314](https://sonarsource.github.io/rspec/#/rspec/S1314/javascript) | [eslint/no-octal](https://eslint.org/docs/latest/rules/no-octal)                                                                                                                               |
| [S1321](https://sonarsource.github.io/rspec/#/rspec/S1321/javascript) | [eslint/no-with](https://eslint.org/docs/latest/rules/no-with)                                                                                                                                 |
| [S1440](https://sonarsource.github.io/rspec/#/rspec/S1440/javascript) | [eslint/eqeqeq](https://eslint.org/docs/latest/rules/eqeqeq)                                                                                                                                   |
| [S1441](https://sonarsource.github.io/rspec/#/rspec/S1441/javascript) | [eslint/quotes](https://eslint.org/docs/latest/rules/quotes)                                                                                                                                   |
| [S1442](https://sonarsource.github.io/rspec/#/rspec/S1442/javascript) | [eslint/no-alert](https://eslint.org/docs/latest/rules/no-alert)                                                                                                                               |
| [S1516](https://sonarsource.github.io/rspec/#/rspec/S1516/javascript) | [eslint/no-multi-str](https://eslint.org/docs/latest/rules/no-multi-str)                                                                                                                       |
| [S1525](https://sonarsource.github.io/rspec/#/rspec/S1525/javascript) | [eslint/no-debugger](https://eslint.org/docs/latest/rules/no-debugger)                                                                                                                         |
| [S1536](https://sonarsource.github.io/rspec/#/rspec/S1536/javascript) | [eslint/no-dupe-args](https://eslint.org/docs/latest/rules/no-dupe-args)                                                                                                                       |
| [S1537](https://sonarsource.github.io/rspec/#/rspec/S1537/javascript) | [typescript-eslint/comma-dangle](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/comma-dangle.mdx)                                       |
| [S1539](https://sonarsource.github.io/rspec/#/rspec/S1539/javascript) | [eslint/strict](https://eslint.org/docs/latest/rules/strict)                                                                                                                                   |
| [S1656](https://sonarsource.github.io/rspec/#/rspec/S1656/javascript) | [eslint/no-self-assign](https://eslint.org/docs/latest/rules/no-self-assign)                                                                                                                   |
| [S1774](https://sonarsource.github.io/rspec/#/rspec/S1774/javascript) | [eslint/no-ternary](https://eslint.org/docs/latest/rules/no-ternary)                                                                                                                           |
| [S2094](https://sonarsource.github.io/rspec/#/rspec/S2094/javascript) | [typescript-eslint/no-extraneous-class](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-extraneous-class.mdx)                         |
| [S2427](https://sonarsource.github.io/rspec/#/rspec/S2427/javascript) | [eslint/radix](https://eslint.org/docs/latest/rules/radix)                                                                                                                                     |
| [S2432](https://sonarsource.github.io/rspec/#/rspec/S2432/javascript) | [eslint/no-setter-return](https://eslint.org/docs/latest/rules/no-setter-return)                                                                                                               |
| [S2685](https://sonarsource.github.io/rspec/#/rspec/S2685/javascript) | [eslint/no-caller](https://eslint.org/docs/latest/rules/no-caller)                                                                                                                             |
| [S2933](https://sonarsource.github.io/rspec/#/rspec/S2933/javascript) | [typescript-eslint/prefer-readonly](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-readonly.mdx)                                 |
| [S2966](https://sonarsource.github.io/rspec/#/rspec/S2966/javascript) | [typescript-eslint/no-non-null-assertion](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-non-null-assertion.mdx)                     |
| [S3257](https://sonarsource.github.io/rspec/#/rspec/S3257/javascript) | [typescript-eslint/no-inferrable-types](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-inferrable-types.mdx)                         |
| [S3353](https://sonarsource.github.io/rspec/#/rspec/S3353/javascript) | [eslint/prefer-const](https://eslint.org/docs/latest/rules/prefer-const)                                                                                                                       |
| [S3523](https://sonarsource.github.io/rspec/#/rspec/S3523/javascript) | [eslint/no-new-func](https://eslint.org/docs/latest/rules/no-new-func)                                                                                                                         |
| [S3786](https://sonarsource.github.io/rspec/#/rspec/S3786/javascript) | [eslint/no-template-curly-in-string](https://eslint.org/docs/latest/rules/no-template-curly-in-string)                                                                                         |
| [S3799](https://sonarsource.github.io/rspec/#/rspec/S3799/javascript) | [eslint/no-empty-pattern](https://eslint.org/docs/latest/rules/no-empty-pattern)                                                                                                               |
| [S3812](https://sonarsource.github.io/rspec/#/rspec/S3812/javascript) | [eslint/no-unsafe-negation](https://eslint.org/docs/latest/rules/no-unsafe-negation)                                                                                                           |
| [S3834](https://sonarsource.github.io/rspec/#/rspec/S3834/javascript) | [eslint/no-new-native-nonconstructor](https://eslint.org/docs/latest/rules/no-new-native-nonconstructor)                                                                                       |
| [S3863](https://sonarsource.github.io/rspec/#/rspec/S3863/javascript) | [import/no-duplicates](https://github.com/import-js/eslint-plugin-import/blob/HEAD/docs/rules/no-duplicates.md)                                                                                |
| [S4124](https://sonarsource.github.io/rspec/#/rspec/S4124/javascript) | [typescript-eslint/no-misused-new](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-misused-new.mdx)                                   |
| [S4125](https://sonarsource.github.io/rspec/#/rspec/S4125/javascript) | [eslint/valid-typeof](https://eslint.org/docs/latest/rules/valid-typeof)                                                                                                                       |
| [S4136](https://sonarsource.github.io/rspec/#/rspec/S4136/javascript) | [typescript-eslint/adjacent-overload-signatures](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/adjacent-overload-signatures.mdx)       |
| [S4137](https://sonarsource.github.io/rspec/#/rspec/S4137/javascript) | [typescript-eslint/consistent-type-assertions](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/consistent-type-assertions.mdx)           |
| [S4140](https://sonarsource.github.io/rspec/#/rspec/S4140/javascript) | [eslint/no-sparse-arrays](https://eslint.org/docs/latest/rules/no-sparse-arrays)                                                                                                               |
| [S4157](https://sonarsource.github.io/rspec/#/rspec/S4157/javascript) | [typescript-eslint/no-unnecessary-type-arguments](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-unnecessary-type-arguments.mdx)     |
| [S4204](https://sonarsource.github.io/rspec/#/rspec/S4204/javascript) | [typescript-eslint/no-explicit-any](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-explicit-any.mdx)                                 |
| [S4325](https://sonarsource.github.io/rspec/#/rspec/S4325/javascript) | [typescript-eslint/no-unnecessary-type-assertion](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-unnecessary-type-assertion.mdx)     |
| [S4326](https://sonarsource.github.io/rspec/#/rspec/S4326/javascript) | [eslint/no-return-await](https://eslint.org/docs/latest/rules/no-return-await)                                                                                                                 |
| [S6325](https://sonarsource.github.io/rspec/#/rspec/S6325/javascript) | [eslint/prefer-regex-literals](https://eslint.org/docs/latest/rules/prefer-regex-literals)                                                                                                     |
| [S6435](https://sonarsource.github.io/rspec/#/rspec/S6435/javascript) | [react/require-render-return](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/require-render-return.md)                                                                 |
| [S6438](https://sonarsource.github.io/rspec/#/rspec/S6438/javascript) | [react/jsx-no-comment-textnodes](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/jsx-no-comment-textnodes.md)                                                           |
| [S6480](https://sonarsource.github.io/rspec/#/rspec/S6480/javascript) | [react/jsx-no-bind](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/jsx-no-bind.md)                                                                                     |
| [S6509](https://sonarsource.github.io/rspec/#/rspec/S6509/javascript) | [eslint/no-extra-boolean-cast](https://eslint.org/docs/latest/rules/no-extra-boolean-cast)                                                                                                     |
| [S6522](https://sonarsource.github.io/rspec/#/rspec/S6522/javascript) | [eslint/no-import-assign](https://eslint.org/docs/latest/rules/no-import-assign)                                                                                                               |
| [S6523](https://sonarsource.github.io/rspec/#/rspec/S6523/javascript) | [eslint/no-unsafe-optional-chaining](https://eslint.org/docs/latest/rules/no-unsafe-optional-chaining)                                                                                         |
| [S6534](https://sonarsource.github.io/rspec/#/rspec/S6534/javascript) | [eslint/no-loss-of-precision](https://eslint.org/docs/latest/rules/no-loss-of-precision)                                                                                                       |
| [S6550](https://sonarsource.github.io/rspec/#/rspec/S6550/javascript) | [typescript-eslint/prefer-literal-enum-member](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-literal-enum-member.mdx)           |
| [S6565](https://sonarsource.github.io/rspec/#/rspec/S6565/javascript) | [typescript-eslint/prefer-return-this-type](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-return-this-type.mdx)                 |
| [S6568](https://sonarsource.github.io/rspec/#/rspec/S6568/javascript) | [typescript-eslint/no-confusing-non-null-assertion](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-confusing-non-null-assertion.mdx) |
| [S6569](https://sonarsource.github.io/rspec/#/rspec/S6569/javascript) | [typescript-eslint/no-unnecessary-type-constraint](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-unnecessary-type-constraint.mdx)   |
| [S6578](https://sonarsource.github.io/rspec/#/rspec/S6578/javascript) | [typescript-eslint/no-duplicate-enum-values](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-duplicate-enum-values.mdx)               |
| [S6583](https://sonarsource.github.io/rspec/#/rspec/S6583/javascript) | [typescript-eslint/no-mixed-enums](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-mixed-enums.mdx)                                   |
| [S6590](https://sonarsource.github.io/rspec/#/rspec/S6590/javascript) | [typescript-eslint/prefer-as-const](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-as-const.mdx)                                 |
| [S6635](https://sonarsource.github.io/rspec/#/rspec/S6635/javascript) | [eslint/no-constructor-return](https://eslint.org/docs/latest/rules/no-constructor-return)                                                                                                     |
| [S6637](https://sonarsource.github.io/rspec/#/rspec/S6637/javascript) | [eslint/no-extra-bind](https://eslint.org/docs/latest/rules/no-extra-bind)                                                                                                                     |
| [S6638](https://sonarsource.github.io/rspec/#/rspec/S6638/javascript) | [eslint/no-constant-binary-expression](https://eslint.org/docs/latest/rules/no-constant-binary-expression)                                                                                     |
| [S6644](https://sonarsource.github.io/rspec/#/rspec/S6644/javascript) | [eslint/no-unneeded-ternary](https://eslint.org/docs/latest/rules/no-unneeded-ternary)                                                                                                         |
| [S6645](https://sonarsource.github.io/rspec/#/rspec/S6645/javascript) | [eslint/no-undef-init](https://eslint.org/docs/latest/rules/no-undef-init)                                                                                                                     |
| [S6650](https://sonarsource.github.io/rspec/#/rspec/S6650/javascript) | [eslint/no-useless-rename](https://eslint.org/docs/latest/rules/no-useless-rename)                                                                                                             |
| [S6653](https://sonarsource.github.io/rspec/#/rspec/S6653/javascript) | [eslint/prefer-object-has-own](https://eslint.org/docs/latest/rules/prefer-object-has-own)                                                                                                     |
| [S6654](https://sonarsource.github.io/rspec/#/rspec/S6654/javascript) | [eslint/no-proto](https://eslint.org/docs/latest/rules/no-proto)                                                                                                                               |
| [S6657](https://sonarsource.github.io/rspec/#/rspec/S6657/javascript) | [eslint/no-octal-escape](https://eslint.org/docs/latest/rules/no-octal-escape)                                                                                                                 |
| [S6671](https://sonarsource.github.io/rspec/#/rspec/S6671/javascript) | [typescript-eslint/prefer-promise-reject-errors](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-promise-reject-errors.mdx)       |
| [S6746](https://sonarsource.github.io/rspec/#/rspec/S6746/javascript) | [react/no-direct-mutation-state](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-direct-mutation-state.md)                                                           |
| [S6748](https://sonarsource.github.io/rspec/#/rspec/S6748/javascript) | [react/no-children-prop](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-children-prop.md)                                                                           |
| [S6750](https://sonarsource.github.io/rspec/#/rspec/S6750/javascript) | [react/no-render-return-value](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-render-return-value.md)                                                               |
| [S6756](https://sonarsource.github.io/rspec/#/rspec/S6756/javascript) | [react/no-access-state-in-setstate](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-access-state-in-setstate.md)                                                     |
| [S6757](https://sonarsource.github.io/rspec/#/rspec/S6757/javascript) | [react/no-this-in-sfc](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-this-in-sfc.md)                                                                               |
| [S6761](https://sonarsource.github.io/rspec/#/rspec/S6761/javascript) | [react/no-danger-with-children](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-danger-with-children.md)                                                             |
| [S6763](https://sonarsource.github.io/rspec/#/rspec/S6763/javascript) | [react/no-redundant-should-component-update](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-redundant-should-component-update.md)                                   |
| [S6766](https://sonarsource.github.io/rspec/#/rspec/S6766/javascript) | [react/no-unescaped-entities](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-unescaped-entities.md)                                                                 |
| [S6767](https://sonarsource.github.io/rspec/#/rspec/S6767/javascript) | [react/no-unused-prop-types](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-unused-prop-types.md)                                                                   |
| [S6770](https://sonarsource.github.io/rspec/#/rspec/S6770/javascript) | [react/jsx-pascal-case](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/jsx-pascal-case.md)                                                                             |
| [S6772](https://sonarsource.github.io/rspec/#/rspec/S6772/javascript) | [react/jsx-child-element-spacing](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/jsx-child-element-spacing.md)                                                         |
| [S6774](https://sonarsource.github.io/rspec/#/rspec/S6774/javascript) | [react/prop-types](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/prop-types.md)                                                                                       |
| [S6775](https://sonarsource.github.io/rspec/#/rspec/S6775/javascript) | [react/default-props-match-prop-types](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/default-props-match-prop-types.md)                                               |
| [S6789](https://sonarsource.github.io/rspec/#/rspec/S6789/javascript) | [react/no-is-mounted](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-is-mounted.md)                                                                                 |
| [S6790](https://sonarsource.github.io/rspec/#/rspec/S6790/javascript) | [react/no-string-refs](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-string-refs.md)                                                                               |
| [S6793](https://sonarsource.github.io/rspec/#/rspec/S6793/javascript) | [jsx-a11y/aria-proptypes](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/aria-proptypes.md)                                                                         |
| [S6807](https://sonarsource.github.io/rspec/#/rspec/S6807/javascript) | [jsx-a11y/role-has-required-aria-props](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/role-has-required-aria-props.md)                                             |
| [S6811](https://sonarsource.github.io/rspec/#/rspec/S6811/javascript) | [jsx-a11y/role-supports-aria-props](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/role-supports-aria-props.md)                                                     |
| [S6819](https://sonarsource.github.io/rspec/#/rspec/S6819/javascript) | [jsx-a11y/prefer-tag-over-role](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/prefer-tag-over-role.md)                                                             |
| [S6821](https://sonarsource.github.io/rspec/#/rspec/S6821/javascript) | [jsx-a11y/aria-role](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/aria-role.md)                                                                                   |
| [S6822](https://sonarsource.github.io/rspec/#/rspec/S6822/javascript) | [jsx-a11y/no-redundant-roles](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/no-redundant-roles.md)                                                                 |
| [S6823](https://sonarsource.github.io/rspec/#/rspec/S6823/javascript) | [jsx-a11y/aria-activedescendant-has-tabindex](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/aria-activedescendant-has-tabindex.md)                                 |
| [S6824](https://sonarsource.github.io/rspec/#/rspec/S6824/javascript) | [jsx-a11y/aria-unsupported-elements](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/aria-unsupported-elements.md)                                                   |
| [S6825](https://sonarsource.github.io/rspec/#/rspec/S6825/javascript) | [jsx-a11y/no-aria-hidden-on-focusable](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/no-aria-hidden-on-focusable.md)                                               |
| [S6836](https://sonarsource.github.io/rspec/#/rspec/S6836/javascript) | [eslint/no-case-declarations](https://eslint.org/docs/latest/rules/no-case-declarations)                                                                                                       |
| [S6840](https://sonarsource.github.io/rspec/#/rspec/S6840/javascript) | [jsx-a11y/autocomplete-valid](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/autocomplete-valid.md)                                                                 |
| [S6841](https://sonarsource.github.io/rspec/#/rspec/S6841/javascript) | [jsx-a11y/tabindex-no-positive](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/tabindex-no-positive.md)                                                             |
| [S6842](https://sonarsource.github.io/rspec/#/rspec/S6842/javascript) | [jsx-a11y/no-noninteractive-element-to-interactive-role](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/no-noninteractive-element-to-interactive-role.md)           |
| [S6843](https://sonarsource.github.io/rspec/#/rspec/S6843/javascript) | [jsx-a11y/no-interactive-element-to-noninteractive-role](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/no-interactive-element-to-noninteractive-role.md)           |
| [S6845](https://sonarsource.github.io/rspec/#/rspec/S6845/javascript) | [jsx-a11y/no-noninteractive-tabindex](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/no-noninteractive-tabindex.md)                                                 |
| [S6846](https://sonarsource.github.io/rspec/#/rspec/S6846/javascript) | [jsx-a11y/no-access-key](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/no-access-key.md)                                                                           |
| [S6847](https://sonarsource.github.io/rspec/#/rspec/S6847/javascript) | [jsx-a11y/no-noninteractive-element-interactions](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/no-noninteractive-element-interactions.md)                         |
| [S6848](https://sonarsource.github.io/rspec/#/rspec/S6848/javascript) | [jsx-a11y/no-static-element-interactions](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/no-static-element-interactions.md)                                         |
| [S6850](https://sonarsource.github.io/rspec/#/rspec/S6850/javascript) | [jsx-a11y/heading-has-content](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/heading-has-content.md)                                                               |
| [S6851](https://sonarsource.github.io/rspec/#/rspec/S6851/javascript) | [jsx-a11y/img-redundant-alt](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/img-redundant-alt.md)                                                                   |
| [S6852](https://sonarsource.github.io/rspec/#/rspec/S6852/javascript) | [jsx-a11y/interactive-supports-focus](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/interactive-supports-focus.md)                                                 |
| [S6859](https://sonarsource.github.io/rspec/#/rspec/S6859/javascript) | [import/no-absolute-path](https://github.com/import-js/eslint-plugin-import/blob/HEAD/docs/rules/no-absolute-path.md)                                                                          |
| [S6861](https://sonarsource.github.io/rspec/#/rspec/S6861/javascript) | [import/no-mutable-exports](https://github.com/import-js/eslint-plugin-import/blob/HEAD/docs/rules/no-mutable-exports.md)                                                                      |

<!--- end external rules -->

## Improved ESLint rules

Also not available in this ESLint plugin, but used in SonarJS, this list of rules have been modified, so your experience using them in your ESLint configuration may differ from the results you may get using SonarQube:

<!--- start decorated rules -->

| SonarJS rule ID                                                       | Rules used in the SonarJS implementation                                                                                                                                                                                                                                                                                                                                                   |
| :-------------------------------------------------------------------- | :----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| [S107](https://sonarsource.github.io/rspec/#/rspec/S107/javascript)   | [eslint/max-params](https://eslint.org/docs/latest/rules/max-params)                                                                                                                                                                                                                                                                                                                       |
| [S109](https://sonarsource.github.io/rspec/#/rspec/S109/javascript)   | [typescript-eslint/no-magic-numbers](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-magic-numbers.mdx)                                                                                                                                                                                                                           |
| [S131](https://sonarsource.github.io/rspec/#/rspec/S131/javascript)   | [typescript-eslint/switch-exhaustiveness-check](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/switch-exhaustiveness-check.mdx)                                                                                                                                                                                                     |
| [S905](https://sonarsource.github.io/rspec/#/rspec/S905/javascript)   | [typescript-eslint/no-unused-expressions](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-unused-expressions.mdx)                                                                                                                                                                                                                 |
| [S1068](https://sonarsource.github.io/rspec/#/rspec/S1068/javascript) | [eslint/no-unused-private-class-members](https://eslint.org/docs/latest/rules/no-unused-private-class-members)                                                                                                                                                                                                                                                                             |
| [S1077](https://sonarsource.github.io/rspec/#/rspec/S1077/javascript) | [jsx-a11y/alt-text](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/alt-text.md)                                                                                                                                                                                                                                                                                 |
| [S1082](https://sonarsource.github.io/rspec/#/rspec/S1082/javascript) | [jsx-a11y/mouse-events-have-key-events](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/mouse-events-have-key-events.md)<br>[jsx-a11y/click-events-have-key-events](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/click-events-have-key-events.md)                                                                                   |
| [S1105](https://sonarsource.github.io/rspec/#/rspec/S1105/javascript) | [eslint/brace-style](https://eslint.org/docs/latest/rules/brace-style)                                                                                                                                                                                                                                                                                                                     |
| [S1116](https://sonarsource.github.io/rspec/#/rspec/S1116/javascript) | [typescript-eslint/no-extra-semi](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-extra-semi.mdx)                                                                                                                                                                                                                                 |
| [S1186](https://sonarsource.github.io/rspec/#/rspec/S1186/javascript) | [typescript-eslint/no-empty-function](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-empty-function.mdx)                                                                                                                                                                                                                         |
| [S1438](https://sonarsource.github.io/rspec/#/rspec/S1438/javascript) | [typescript-eslint/semi](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/semi.mdx)                                                                                                                                                                                                                                                   |
| [S1534](https://sonarsource.github.io/rspec/#/rspec/S1534/javascript) | [eslint/no-dupe-keys](https://eslint.org/docs/latest/rules/no-dupe-keys)<br>[typescript-eslint/no-dupe-class-members](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-dupe-class-members.mdx)<br>[react/jsx-no-duplicate-props](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/jsx-no-duplicate-props.md) |
| [S1763](https://sonarsource.github.io/rspec/#/rspec/S1763/javascript) | [eslint/no-unreachable](https://eslint.org/docs/latest/rules/no-unreachable)                                                                                                                                                                                                                                                                                                               |
| [S1788](https://sonarsource.github.io/rspec/#/rspec/S1788/javascript) | [typescript-eslint/default-param-last](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/default-param-last.mdx)                                                                                                                                                                                                                       |
| [S2189](https://sonarsource.github.io/rspec/#/rspec/S2189/javascript) | [eslint/no-unmodified-loop-condition](https://eslint.org/docs/latest/rules/no-unmodified-loop-condition)                                                                                                                                                                                                                                                                                   |
| [S2376](https://sonarsource.github.io/rspec/#/rspec/S2376/javascript) | [eslint/accessor-pairs](https://eslint.org/docs/latest/rules/accessor-pairs)                                                                                                                                                                                                                                                                                                               |
| [S2430](https://sonarsource.github.io/rspec/#/rspec/S2430/javascript) | [eslint/new-cap](https://eslint.org/docs/latest/rules/new-cap)                                                                                                                                                                                                                                                                                                                             |
| [S2688](https://sonarsource.github.io/rspec/#/rspec/S2688/javascript) | [eslint/use-isnan](https://eslint.org/docs/latest/rules/use-isnan)                                                                                                                                                                                                                                                                                                                         |
| [S2814](https://sonarsource.github.io/rspec/#/rspec/S2814/javascript) | [typescript-eslint/no-redeclare](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-redeclare.mdx)                                                                                                                                                                                                                                   |
| [S3498](https://sonarsource.github.io/rspec/#/rspec/S3498/javascript) | [eslint/object-shorthand](https://eslint.org/docs/latest/rules/object-shorthand)                                                                                                                                                                                                                                                                                                           |
| [S3504](https://sonarsource.github.io/rspec/#/rspec/S3504/javascript) | [eslint/no-var](https://eslint.org/docs/latest/rules/no-var)                                                                                                                                                                                                                                                                                                                               |
| [S3512](https://sonarsource.github.io/rspec/#/rspec/S3512/javascript) | [eslint/prefer-template](https://eslint.org/docs/latest/rules/prefer-template)                                                                                                                                                                                                                                                                                                             |
| [S3696](https://sonarsource.github.io/rspec/#/rspec/S3696/javascript) | [eslint/no-throw-literal](https://eslint.org/docs/latest/rules/no-throw-literal)                                                                                                                                                                                                                                                                                                           |
| [S4023](https://sonarsource.github.io/rspec/#/rspec/S4023/javascript) | [typescript-eslint/no-empty-interface](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-empty-interface.mdx)                                                                                                                                                                                                                       |
| [S4084](https://sonarsource.github.io/rspec/#/rspec/S4084/javascript) | [jsx-a11y/media-has-caption](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/media-has-caption.md)                                                                                                                                                                                                                                                               |
| [S4138](https://sonarsource.github.io/rspec/#/rspec/S4138/javascript) | [typescript-eslint/prefer-for-of](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-for-of.mdx)                                                                                                                                                                                                                                 |
| [S4156](https://sonarsource.github.io/rspec/#/rspec/S4156/javascript) | [typescript-eslint/prefer-namespace-keyword](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-namespace-keyword.mdx)                                                                                                                                                                                                           |
| [S4275](https://sonarsource.github.io/rspec/#/rspec/S4275/javascript) | [eslint/getter-return](https://eslint.org/docs/latest/rules/getter-return)                                                                                                                                                                                                                                                                                                                 |
| [S4327](https://sonarsource.github.io/rspec/#/rspec/S4327/javascript) | [typescript-eslint/no-this-alias](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-this-alias.mdx)                                                                                                                                                                                                                                 |
| [S5254](https://sonarsource.github.io/rspec/#/rspec/S5254/javascript) | [jsx-a11y/lang](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/lang.md)<br>[jsx-a11y/html-has-lang](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/html-has-lang.md)                                                                                                                                                                 |
| [S6440](https://sonarsource.github.io/rspec/#/rspec/S6440/javascript) | [react-hooks/rules-of-hooks](https://github.com/facebook/react/blob/main/packages/eslint-plugin-react-hooks/README.md)                                                                                                                                                                                                                                                                     |
| [S6441](https://sonarsource.github.io/rspec/#/rspec/S6441/javascript) | [react/no-unused-class-component-methods](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-unused-class-component-methods.md)                                                                                                                                                                                                                                     |
| [S6477](https://sonarsource.github.io/rspec/#/rspec/S6477/javascript) | [react/jsx-key](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/jsx-key.md)                                                                                                                                                                                                                                                                                         |
| [S6478](https://sonarsource.github.io/rspec/#/rspec/S6478/javascript) | [react/no-unstable-nested-components](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-unstable-nested-components.md)                                                                                                                                                                                                                                             |
| [S6479](https://sonarsource.github.io/rspec/#/rspec/S6479/javascript) | [react/no-array-index-key](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-array-index-key.md)                                                                                                                                                                                                                                                                   |
| [S6481](https://sonarsource.github.io/rspec/#/rspec/S6481/javascript) | [react/jsx-no-constructed-context-values](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/jsx-no-constructed-context-values.md)                                                                                                                                                                                                                                     |
| [S6535](https://sonarsource.github.io/rspec/#/rspec/S6535/javascript) | [eslint/no-useless-escape](https://eslint.org/docs/latest/rules/no-useless-escape)<br>[eslint/no-nonoctal-decimal-escape](https://eslint.org/docs/latest/rules/no-nonoctal-decimal-escape)                                                                                                                                                                                                 |
| [S6544](https://sonarsource.github.io/rspec/#/rspec/S6544/javascript) | [typescript-eslint/no-misused-promises](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-misused-promises.mdx)<br>[eslint/no-async-promise-executor](https://eslint.org/docs/latest/rules/no-async-promise-executor)                                                                                                               |
| [S6551](https://sonarsource.github.io/rspec/#/rspec/S6551/javascript) | [typescript-eslint/no-base-to-string](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-base-to-string.mdx)                                                                                                                                                                                                                         |
| [S6557](https://sonarsource.github.io/rspec/#/rspec/S6557/javascript) | [typescript-eslint/prefer-string-starts-ends-with](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-string-starts-ends-with.mdx)                                                                                                                                                                                               |
| [S6571](https://sonarsource.github.io/rspec/#/rspec/S6571/javascript) | [typescript-eslint/no-redundant-type-constituents](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/no-redundant-type-constituents.mdx)                                                                                                                                                                                               |
| [S6572](https://sonarsource.github.io/rspec/#/rspec/S6572/javascript) | [typescript-eslint/prefer-enum-initializers](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-enum-initializers.mdx)                                                                                                                                                                                                           |
| [S6582](https://sonarsource.github.io/rspec/#/rspec/S6582/javascript) | [typescript-eslint/prefer-optional-chain](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-optional-chain.mdx)                                                                                                                                                                                                                 |
| [S6598](https://sonarsource.github.io/rspec/#/rspec/S6598/javascript) | [typescript-eslint/prefer-function-type](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-function-type.mdx)                                                                                                                                                                                                                   |
| [S6606](https://sonarsource.github.io/rspec/#/rspec/S6606/javascript) | [typescript-eslint/prefer-nullish-coalescing](https://github.com/typescript-eslint/typescript-eslint/blob/v7.18.0/packages/eslint-plugin/docs/rules/prefer-nullish-coalescing.mdx)                                                                                                                                                                                                         |
| [S6643](https://sonarsource.github.io/rspec/#/rspec/S6643/javascript) | [eslint/no-extend-native](https://eslint.org/docs/latest/rules/no-extend-native)                                                                                                                                                                                                                                                                                                           |
| [S6647](https://sonarsource.github.io/rspec/#/rspec/S6647/javascript) | [eslint/no-useless-constructor](https://eslint.org/docs/latest/rules/no-useless-constructor)                                                                                                                                                                                                                                                                                               |
| [S6660](https://sonarsource.github.io/rspec/#/rspec/S6660/javascript) | [eslint/no-lonely-if](https://eslint.org/docs/latest/rules/no-lonely-if)                                                                                                                                                                                                                                                                                                                   |
| [S6661](https://sonarsource.github.io/rspec/#/rspec/S6661/javascript) | [eslint/prefer-object-spread](https://eslint.org/docs/latest/rules/prefer-object-spread)                                                                                                                                                                                                                                                                                                   |
| [S6666](https://sonarsource.github.io/rspec/#/rspec/S6666/javascript) | [eslint/prefer-spread](https://eslint.org/docs/latest/rules/prefer-spread)                                                                                                                                                                                                                                                                                                                 |
| [S6676](https://sonarsource.github.io/rspec/#/rspec/S6676/javascript) | [eslint/no-useless-call](https://eslint.org/docs/latest/rules/no-useless-call)                                                                                                                                                                                                                                                                                                             |
| [S6679](https://sonarsource.github.io/rspec/#/rspec/S6679/javascript) | [eslint/no-self-compare](https://eslint.org/docs/latest/rules/no-self-compare)                                                                                                                                                                                                                                                                                                             |
| [S6747](https://sonarsource.github.io/rspec/#/rspec/S6747/javascript) | [react/no-unknown-property](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-unknown-property.md)<br>[jsx-a11y/aria-props](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/aria-props.md)                                                                                                                                               |
| [S6749](https://sonarsource.github.io/rspec/#/rspec/S6749/javascript) | [react/jsx-no-useless-fragment](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/jsx-no-useless-fragment.md)                                                                                                                                                                                                                                                         |
| [S6754](https://sonarsource.github.io/rspec/#/rspec/S6754/javascript) | [react/hook-use-state](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/hook-use-state.md)                                                                                                                                                                                                                                                                           |
| [S6788](https://sonarsource.github.io/rspec/#/rspec/S6788/javascript) | [react/no-find-dom-node](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-find-dom-node.md)                                                                                                                                                                                                                                                                       |
| [S6791](https://sonarsource.github.io/rspec/#/rspec/S6791/javascript) | [react/no-unsafe](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-unsafe.md)                                                                                                                                                                                                                                                                                     |
| [S6827](https://sonarsource.github.io/rspec/#/rspec/S6827/javascript) | [jsx-a11y/anchor-has-content](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/anchor-has-content.md)                                                                                                                                                                                                                                                             |
| [S6844](https://sonarsource.github.io/rspec/#/rspec/S6844/javascript) | [jsx-a11y/anchor-is-valid](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/anchor-is-valid.md)                                                                                                                                                                                                                                                                   |
| [S6853](https://sonarsource.github.io/rspec/#/rspec/S6853/javascript) | [jsx-a11y/label-has-associated-control](https://github.com/jsx-eslint/eslint-plugin-jsx-a11y/blob/HEAD/docs/rules/label-has-associated-control.md)                                                                                                                                                                                                                                         |
| [S6957](https://sonarsource.github.io/rspec/#/rspec/S6957/javascript) | [react/no-deprecated](https://github.com/jsx-eslint/eslint-plugin-react/blob/HEAD/docs/rules/no-deprecated.md)                                                                                                                                                                                                                                                                             |
| [S7060](https://sonarsource.github.io/rspec/#/rspec/S7060/javascript) | [import/no-self-import](https://github.com/import-js/eslint-plugin-import/blob/HEAD/docs/rules/no-self-import.md)                                                                                                                                                                                                                                                                          |

<!--- end decorated rules -->
